@@ -3,13 +3,20 @@ from __future__ import annotations
 import asyncio
 import logging
 from typing import (
+    Any,
     AsyncGenerator,
-    ClassVar, Type,
-    Optional, Tuple, Union,
+    ClassVar,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    Union,
 )
+import secrets
+import warnings
 
 import attr
-import zmq, zmq.asyncio
+import zmq, zmq.asyncio, zmq.utils.monitor
 
 from ..abc import RawHeaderBody
 from ..auth import Identity
@@ -22,6 +29,10 @@ from . import (
 
 ZAP_VERSION = b'1.0'
 
+_default_zsock_opts = {
+    zmq.LINGER: 100,
+}
+
 
 @attr.s(auto_attribs=True, slots=True)
 class ZeroMQAddress(AbstractAddress):
@@ -136,6 +147,7 @@ class ZeroMQRPCConnection(AbstractConnection):
 
     async def recv_message(self) -> AsyncGenerator[Optional[RawHeaderBody], None]:
         assert not self.transport._closed
+        assert self.transport._sock is not None
         *pre, raw_header, raw_body = await self.transport._sock.recv_multipart()
         if len(pre) > 0:
             # server
@@ -147,6 +159,7 @@ class ZeroMQRPCConnection(AbstractConnection):
 
     async def send_message(self, raw_msg: RawHeaderBody) -> None:
         assert not self.transport._closed
+        assert self.transport._sock is not None
         peer_id = raw_msg.peer_id
         if peer_id is not None:
             # server
@@ -160,15 +173,88 @@ class ZeroMQRPCConnection(AbstractConnection):
             ])
 
 
-class ZeroMQBaseBinder(AbstractBinder):
+class ZeroMQMonitorMixin:
 
-    __slots__ = ('transport', 'addr')
+    addr: ZeroMQAddress
+
+    _monitor_sock: Optional[zmq.Socket]
+
+    EVENT_MAP = {
+        zmq.EVENT_MONITOR_STOPPED: "monitor-stopped",
+        zmq.EVENT_ACCEPT_FAILED: "accept-failed",
+        zmq.EVENT_ACCEPTED: "accepted",
+        zmq.EVENT_BIND_FAILED: "bind-failed",
+        zmq.EVENT_LISTENING: "listening",
+        zmq.EVENT_CLOSE_FAILED: "close-failed",
+        zmq.EVENT_CLOSED: "closed",
+        zmq.EVENT_CONNECTED: "connected",
+        zmq.EVENT_CONNECT_DELAYED: "connect-delayed",
+        zmq.EVENT_CONNECT_RETRIED: "connect-retried",
+        zmq.EVENT_DISCONNECTED: "disconnected",
+        zmq.EVENT_HANDSHAKE_FAILED_AUTH: "handshake-faield-auth",
+        zmq.EVENT_HANDSHAKE_FAILED_NO_DETAIL: "handshake-failed-no-detail",
+        zmq.EVENT_HANDSHAKE_FAILED_PROTOCOL: "handshake-failed-protocol",
+        zmq.EVENT_HANDSHAKE_SUCCEEDED: "handshake-succeeded",
+        zmq.EVENT_ALL: "all",
+    }
+
+    async def _monitor(self) -> None:
+        assert self._monitor_sock is not None
+        log = logging.getLogger('callosum.lower.zeromq.monitor')
+        try:
+            while (await self._monitor_sock.poll()):
+                raw_msg = await self._monitor_sock.recv_multipart()
+                msg = zmq.utils.monitor.parse_monitor_message(raw_msg)
+                msg['description'] = self.EVENT_MAP.get(
+                    msg['event'],
+                    str(msg['event']),
+                )
+                log.debug("monitor[%s] event: %r", self.addr, msg)
+                if msg['event'] == zmq.EVENT_MONITOR_STOPPED:
+                    break
+        except Exception:
+            log.exception("monitor[%s] unexpected error", self.addr)
+        finally:
+            self._monitor_sock.close()
+            log.debug("monitor[%s] closed", self.addr)
+
+
+class ZeroMQBaseBinder(ZeroMQMonitorMixin, AbstractBinder):
+
+    __slots__ = (
+        'transport',
+        'addr',
+        '_attach_monitor',
+        '_monitor_sock',
+        '_monitor_task',
+        '_main_sock',
+        '_zsock_opts',
+    )
 
     socket_type: ClassVar[int] = 0
 
     transport: ZeroMQBaseTransport
     addr: ZeroMQAddress
 
+    def __init__(
+        self,
+        transport: BaseTransport,
+        addr: AbstractAddress,
+        *,
+        attach_monitor: bool = False,
+        zsock_opts: Mapping[int, Any] = None,
+        **transport_opts,
+    ) -> None:
+        super().__init__(transport, addr)
+        self._attach_monitor = attach_monitor
+        self._zsock_opts = {**_default_zsock_opts, **(zsock_opts or {})}
+        if attach_monitor:
+            warnings.warn(
+                "ZeroMQ async monitor socket support is buggy "
+                "and not recommended to use.",
+                RuntimeWarning,
+            )
+
     async def __aenter__(self):
         if not self.transport._closed:
             return ZeroMQRPCConnection(self.transport)
@@ -178,25 +264,64 @@ class ZeroMQBaseBinder(AbstractBinder):
             server_sock.zap_domain = server_id.domain.encode('utf8')
             server_sock.setsockopt(zmq.CURVE_SERVER, 1)
             server_sock.setsockopt(zmq.CURVE_SECRETKEY, server_id.private_key)
-        for key, value in self.transport._zsock_opts.items():
+        for key, value in self._zsock_opts.items():
             server_sock.setsockopt(key, value)
+        if self._attach_monitor:
+            monitor_addr = f"inproc://monitor-{secrets.token_hex(16)}"
+            server_sock.get_monitor_socket(addr=monitor_addr)
+            self._monitor_sock = self.transport._zctx.socket(zmq.PAIR)
+            self._monitor_sock.connect(monitor_addr)
+            self._monitor_task = asyncio.create_task(self._monitor())
+        else:
+            self._monitor_sock = None
+            self._monitor_task = None
         server_sock.bind(self.addr.uri)
         self.transport._sock = server_sock
+        self._main_sock = server_sock
         return ZeroMQRPCConnection(self.transport)
 
     async def __aexit__(self, exc_type, exc_obj, exc_tb):
-        pass
+        if self._monitor_task is not None:
+            self._main_sock.disable_monitor()
+            await self._monitor_task
 
 
-class ZeroMQBaseConnector(AbstractConnector):
+class ZeroMQBaseConnector(ZeroMQMonitorMixin, AbstractConnector):
 
-    __slots__ = ('transport', 'addr')
+    __slots__ = (
+        'transport',
+        'addr',
+        '_attach_monitor',
+        '_monitor_sock',
+        '_monitor_task',
+        '_main_sock',
+        '_zsock_opts',
+    )
 
     socket_type: ClassVar[int] = 0
 
     transport: ZeroMQBaseTransport
     addr: ZeroMQAddress
 
+    def __init__(
+        self,
+        transport: BaseTransport,
+        addr: AbstractAddress,
+        *,
+        attach_monitor: bool = False,
+        zsock_opts: Mapping[int, Any] = None,
+        **transport_opts,
+    ) -> None:
+        super().__init__(transport, addr)
+        self._attach_monitor = attach_monitor
+        self._zsock_opts = {**_default_zsock_opts, **(zsock_opts or {})}
+        if attach_monitor:
+            warnings.warn(
+                "ZeroMQ async monitor socket support is buggy "
+                "and not recommended to use.",
+                RuntimeWarning,
+            )
+
     async def __aenter__(self):
         if not self.transport._closed:
             return ZeroMQRPCConnection(self.transport)
@@ -210,14 +335,26 @@ class ZeroMQBaseConnector(AbstractConnector):
             client_sock.setsockopt(zmq.CURVE_SERVERKEY, server_public_key)
             client_sock.setsockopt(zmq.CURVE_PUBLICKEY, client_public_key)
             client_sock.setsockopt(zmq.CURVE_SECRETKEY, client_id.private_key)
-        for key, value in self.transport._zsock_opts.items():
+        for key, value in self._zsock_opts.items():
             client_sock.setsockopt(key, value)
+        if self._attach_monitor:
+            monitor_addr = f"inproc://monitor-{secrets.token_hex(16)}"
+            client_sock.get_monitor_socket(addr=monitor_addr)
+            self._monitor_sock = self.transport._zctx.socket(zmq.PAIR)
+            self._monitor_sock.connect(monitor_addr)
+            self._monitor_task = asyncio.create_task(self._monitor())
+        else:
+            self._monitor_sock = None
+            self._monitor_task = None
         client_sock.connect(self.addr.uri)
+        self._main_sock = client_sock
         self.transport._sock = client_sock
         return ZeroMQRPCConnection(self.transport)
 
     async def __aexit__(self, exc_type, exc_obj, exc_tb):
-        pass
+        if self._monitor_task is not None:
+            self._main_sock.disable_monitor()
+            await self._monitor_task
 
 
 class ZeroMQRouterBinder(ZeroMQBaseBinder):
@@ -275,30 +412,25 @@ class ZeroMQBaseTransport(BaseTransport):
     binder_cls: ClassVar[Type[ZeroMQBaseBinder]]
     connector_cls: ClassVar[Type[ZeroMQBaseConnector]]
 
-    def __init__(self, authenticator, **kwargs):
+    def __init__(self, authenticator, **kwargs) -> None:
+        super().__init__(authenticator, **kwargs)
         loop = asyncio.get_running_loop()
         self._zap_server = None
         self._zap_task = None
-        transport_opts = kwargs.pop('transport_opts', {})
-        self._zsock_opts = {
-            zmq.LINGER: 100,
-            **transport_opts.get('zsock_opts', {}),
-        }
-        super().__init__(authenticator, **kwargs)
         if self.authenticator:
             self._zap_server = ZAPServer(self.authenticator)
-            self._zap_task = loop.create_task(self.authenticator.serve())
+            self._zap_task = loop.create_task(self._zap_server.serve())
         self._zctx = zmq.asyncio.Context()
         # Keep sockets during the transport lifetime.
         self._sock = None
 
     @property
-    def _closed(self):
+    def _closed(self) -> bool:
         if self._sock is not None:
             return self._sock.closed
         return True
 
-    async def close(self):
+    async def close(self) -> None:
         if self._sock is not None:
             self._sock.close()
         if self._zap_task is not None:
