@@ -1,8 +1,10 @@
-from .message import StreamMessage
 from .channel import Publisher, Consumer
+from .message import StreamMessage
+from .types import ConsumerCallback
 
 __all__ = (
     'StreamMessage',
     'Publisher',
     'Consumer',
+    'ConsumerCallback',
 )
